"""Classes that facilitate moving data from one store into another"""
from mercury_ml.common.utils import singleton

@singleton
class S3Singleton:
    def __init__(self, **kwargs):
        import boto3
        session = boto3.Session(**kwargs)
        self.s3 = session.resource("s3")



def store_dict_on_mongo(data, document_id, database_name, collection_name, mongo_client_params, document_key="",
                        document_key_separator="/", reuse_existing=True, overwrite=True):
    """
    Takes a dictionary input and stores it in an appropriate key in MongoDB

    :param dict data: A dictionary with the data to be stored
    :param document_id: The unique id, corresponding to the _id of the documented to be upserted (updated / inserted to) is found
    :param string database_name: The name of the database where the document to be upserted to is found
    :param string collection_name: The name of the collection where the document to be upserted to is found
    :param dict mongo_client_params: Paramters to be passed to the MongoClient constructor
    :param string document_key: a string representing the key where the data should be stored. For example, a key
    "metrics/test" will store will store the data in the format:
    {
        "_id": document_id
        "metrics" : {
            "test": data
        }
    }
    :param string document_key_separator: The separator with which to parse the document_key. Default "/"
    :param bool reuse_existing: If False a new MongoClient connection will be created every time the function is called.
    If True an existing connection will be reused.

    :return:
    """
    if not reuse_existing:
        from pymongo import MongoClient
        mongo_client = MongoClient(**mongo_client_params)
    else:
        from mercury_ml.common.artifact_storage import MongoClientSingleton
        mongo_client = MongoClientSingleton(**mongo_client_params).client

    mongo_db = getattr(mongo_client, database_name)
    mongo_collection = getattr(mongo_db, collection_name)

    document_key = document_key.replace(document_key_separator, ".")

    if overwrite or not _document_exists(mongo_collection, document_id):
        mongo_collection.update_one(
            {"_id": document_id},
            {"$set": {document_key: data}},
            upsert=True
        )
    else:
        print("Document with _id {} in collection {} on database {} already exists and 'overwrite' has been set to False. Nothing will be written to MongoDB".format(document_id, collection_name, database_name))


def _document_exists(mongo_collection, document_id):
    cursor = mongo_collection.find({document_id: {"$exists": True}}).limit(1)

    if cursor.count() > 0:
        return True
    else:
        return False



import json
import os
import shutil

def save_keras_hdf5(model, filename, local_dir, extension=None):
    """
    Saves a Keras model in .h5 format

    :param model: A Keras model
    :param string local_dir: Local directory where the model is to be saved
    :param string filename: Filename with which the model is to be saved
    :param string extension: Extension to the filename with which the model is to be saved
    :return: The filepath where the model is saved
    """

    if not extension:
        extension = ".h5"

    _make_dirs(local_dir)
    filename = filename + extension
    path = os.path.join(local_dir + "/" + filename)
    model.save(path)

    return path


def save_tensorflow_saved_model_archived(model, local_dir, filename, extension=".zip", model_number="1", temp_base_dir=None):

    if not temp_base_dir:
        temp_base_dir = os.path.join(os.getcwd(), "_tmp_model", filename)

    root_dir=save_tensorflow_saved_model(model=model,
                                         local_dir=temp_base_dir,
                                         filename=filename,
                                         model_number=model_number)

    if not os.path.isdir(local_dir):
        os.makedirs(local_dir)

    shutil.make_archive(
        base_name=os.path.join(local_dir, filename),
        format=extension[1:],
        root_dir=root_dir,
        base_dir=model_number)

    shutil.rmtree(root_dir, ignore_errors=True)

    return os.path.join(local_dir, filename + extension)


def save_tensorflow_saved_model(model, local_dir, filename, model_number="1"):

    from tensorflow.saved_model import save

    root_dir = os.path.join(local_dir, filename)
    sub_dir = os.path.join(root_dir, model_number)

    if not os.path.isdir(sub_dir):
        os.makedirs(sub_dir)

    save(model, sub_dir)

    return root_dir

def _make_dirs(dir):
    import os
    if not os.path.exists(dir):
        os.makedirs(dir)



def get_or_create_spark_session(spark_config=None, log_level="ERROR", enableHiveSupport=True):
    """
    Gets or initiates an Spark session.

    :param dict spark_config: the parameters based on which the Spark session is to be initiated
    :param log_level: The log level of the Spark session
    :param bool enableHiveSupport: If set to true Hive support will be enabled
    :return: A spark session object
    """

    from pyspark.sql import SparkSession

    builder = SparkSession.builder
    if spark_config:
        # http://spark.apache.org/docs/latest/configuration.html
        for key, value in spark_config.items():
            builder = SparkSession.builder.config(key, value)

    if enableHiveSupport:
        builder = builder.enableHiveSupport()

    spark = builder.getOrCreate()
    spark.sparkContext.setLogLevel(log_level)

    #print(spark.sparkContext.getConf().getAll())
    return spark

